/**
 * Abstract service
 * @class ApService
 */
'use strict'

const scopeMixin = require('./mixins/scope_mixin')

/** @lends ApService */
class ApService {
  constructor (store) {
    const s = this
    s.store = store
  }

  /**
   * Get store values
   * @param {...string} keys - Keys to dig
   * @returns {*} - Stored value
   */
  get (...keys) {
    const s = this
    let { store } = s
    let state = store.getState()
    return keys
      .filter((key) => typeof key !== 'undefined')
      .reduce((state, key) => state[ key ], state)
  }

  /**
   * Dispatch an action
   * @param {string} actionType - Action type to dispatch
   * @param {string} fieldName - Field name to dispath
   * @param {*} value - Value to dispatch
   * @param {boolean} [isError=false] - Is error or not
   */
  dispatch (actionType, fieldName, value, isError = false) {
    const s = this
    let { store } = s
    store.dispatch({
      type: actionType,
      meta: { field: fieldName },
      payload: value,
      error: isError
    })
  }
}

module.exports = scopeMixin(ApService)
